@@ -5,13 +5,8 @@
  *
  * The library is free for all purposes without any express
  * guarantee it works.
- *
- * Tom St Denis, tomstdenis@gmail.com, http://libtom.org
  */
 
-/* Implements ECC over Z/pZ for curve y^2 = x^3 + a*x + b
- *
- */
 #include "tomcrypt.h"
 
 /**
@@ -28,15 +23,14 @@
 */
 int ecc_get_size(ecc_key *key)
 {
-   LTC_ARGCHK(key != NULL);
-   if (ltc_ecc_is_valid_idx(key->idx))
-      return key->dp->size;
-   else
-      return INT_MAX; /* large value known to cause it to fail when passed to ecc_make_key() */
+   if (key == NULL) {
+      return INT_MAX;
+   }
+   return key->dp.size;
 }
 
 #endif
-/* $Source$ */
-/* $Revision$ */
-/* $Date$ */
+/* ref:         $Format:%D$ */
+/* git commit:  $Format:%H$ */
+/* commit time: $Format:%ai$ */
 
